/*
 * DialGuard
 * Version 1.0
 * Copyright (C) 2000
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License; or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You can obtain a copy of the GNU General Public License from their
 * web site at http://www.gnu.org/copyleft/gpl.html -- or, you can 
 * write to the Free Software Foundation, Inc., 59 Temple Place - 
 * Suite 330, Boston, MA 02111-1307, USA.
 */ 
/*
 * This is the 'host' program for the "parasite" that actually does the
 * real work.  Other than as an example of Toolbox icon programming, 
 * there is nothing to see here.
 */
#include <windows.h>
#include <winuser.h>
#include <shellapi.h>
#include <commdlg.h>
#include <commctrl.h>

#define IDC_TRAYICON 6000
#define IDM_ABOUT    6001
#define IDM_EXIT     6002
#define MYWM_TRAY  (WM_USER + 1)

const char * lpszAppName = "DialGuard";
const char * VersionString = "DialGuard/Win32 Version 1.0\rReleased under GPL";
const char * sharedString = "DialGuard Shared Mapping";
const char * Warning = "Careful!\rIf you exit now, your system could be"
    "\rleft in an unstable state!\nPlease close all apps, before "
    "clicking (yes) here.";

HMENU IconMenu;
DWORD * lpCount;
extern void StaticLoad(void);

LRESULT CALLBACK wndproc(HWND hwnd ,UINT umsg ,WPARAM word ,LPARAM dword ) {
    DWORD pos;
    POINT pt;

    if ((umsg == MYWM_TRAY) && 
	(word == (UINT)IDC_TRAYICON)) {
       
	if ((dword == WM_RBUTTONDOWN) || 
            (dword == WM_LBUTTONDOWN)) {
            /* WORKAROUND */
            SetForegroundWindow(hwnd);
            /**/
	    if (GetCursorPos(&pt) == 0) {
		/* This isn't as reliable, but it's better than nothing
		 * .. positions tend to be all over the bottom of the screen,
		 * depending on how many times this is called! */
		pos = GetMessagePos();
		pt.x = LOWORD(pos);
		pt.y = HIWORD(pos);
	    }
	    TrackPopupMenu(IconMenu , 
		TPM_BOTTOMALIGN | TPM_LEFTALIGN | TPM_RIGHTBUTTON,
		pt.x ,pt.y ,0 ,hwnd ,NULL );
             /* WORKAROUND */
             PostMessage(hwnd, WM_NULL, 0, 0);
             /**/
	}
    } else if (umsg == WM_COMMAND) {
	if (word == IDM_ABOUT) {
	    MessageBox(hwnd, VersionString, lpszAppName, MB_OK);
	} else if (word == IDM_EXIT) {
            if (*lpCount == 0) {
                PostQuitMessage(0);
            } else if (MessageBox(NULL,Warning, lpszAppName ,
                MB_YESNO|MB_ICONWARNING) == IDYES){ 
                PostQuitMessage(0);
            }
	}
    } else {
	return DefWindowProc(hwnd ,umsg ,word ,dword );
    }
    return 0;
}

void WinMainCRTStartup( ) {
    MSG msg;
    HINSTANCE hInst;
    NOTIFYICONDATA tnd;
    int res;
    HWND hwnd;
    WNDCLASSEX wcex;
    HANDLE hfile;
    
    hInst = GetModuleHandle(NULL);
    if (!hInst) ExitProcess(-1);

    /* Create the shared region for communication */
    hfile = CreateFileMapping((HANDLE)0xFFFFFFFF ,NULL ,PAGE_READWRITE ,
        0 ,4096 ,sharedString );
    lpCount = (DWORD *)MapViewOfFile(hfile ,FILE_MAP_ALL_ACCESS ,0 ,0 ,4);
    *lpCount = 0;

    /* This is required to define a callback, which is necessary
     * for the tray icon to work right.  Oh well. */

    wcex.cbSize = sizeof(WNDCLASSEX);
    wcex.style = 0;
    wcex.lpfnWndProc = (WNDPROC)wndproc;
    wcex.cbClsExtra = 0;
    wcex.cbWndExtra = 0;
    wcex.hInstance = hInst;
    wcex.hIcon = LoadIcon(hInst ,MAKEINTRESOURCE(IDC_TRAYICON));
    if (!wcex.hIcon) goto cantload;
    wcex.hCursor = LoadCursor(NULL, IDC_ARROW);
    wcex.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wcex.lpszMenuName = lpszAppName;
    wcex.lpszClassName = lpszAppName;
    wcex.hIconSm = LoadIcon(hInst ,MAKEINTRESOURCE(IDC_TRAYICON));
    if (!wcex.hIconSm) goto cantload;
    if (RegisterClassEx(&wcex) == 0) goto cantload;

    /* MENU setup */

    IconMenu = CreatePopupMenu();
    AppendMenu(IconMenu ,MFT_STRING ,IDM_ABOUT ,"&About DialGuard");
    AppendMenu(IconMenu ,MFT_SEPARATOR ,0 ,NULL );
    AppendMenu(IconMenu ,MFT_STRING ,IDM_EXIT ,"E&xit" );

    /* Simplest possible "window */

    hwnd = CreateWindow(lpszAppName ,"DialGuard" ,0 ,CW_USEDEFAULT ,
     CW_USEDEFAULT ,CW_USEDEFAULT ,CW_USEDEFAULT ,NULL ,NULL ,hInst, NULL);
    if (!hwnd) goto cantload;

    /* Put the icon in the tray via the ShellAPI */

    tnd.cbSize = sizeof(NOTIFYICONDATA);
    tnd.hWnd = hwnd;
    tnd.uID = (UINT)IDC_TRAYICON;
    tnd.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
    tnd.uCallbackMessage = MYWM_TRAY;
    tnd.hIcon = wcex.hIconSm; 
    lstrcpyn(tnd.szTip, "DialGuard", sizeof(tnd.szTip));

    if (Shell_NotifyIcon(NIM_ADD, &tnd) == 0) goto cantload;
   
    StaticLoad();

    /* And finally, a message loop */
    while (1) {
	res = GetMessage(&msg, NULL, 0, 0 );
	if ((res == -1) || (res == FALSE))
	    break;
	TranslateMessage(&msg);
	DispatchMessage(&msg);
    } 
    Shell_NotifyIcon(NIM_DELETE, &tnd);
    DestroyMenu(IconMenu);
    ExitProcess( 0 );
cantload:
    MessageBox(NULL,"Win32 API error in DialGuard setup!\r"
        "Please contact the author with the details of your setup\r"
        "\rSorry about this." ,"DialGuard" , MB_ICONERROR | MB_OK );
    ExitProcess(-1);
        
}
